package lcsb.mapviewer.services.impl;

import java.util.List;

import org.hibernate.Hibernate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.model.plugin.Plugin;
import lcsb.mapviewer.model.plugin.PluginDataEntry;
import lcsb.mapviewer.model.user.User;
import lcsb.mapviewer.persist.dao.plugin.PluginDao;
import lcsb.mapviewer.persist.dao.plugin.PluginDataEntryDao;
import lcsb.mapviewer.services.interfaces.IPluginService;

@Transactional
@Service
public class PluginService implements IPluginService {

  private PluginDao pluginDao;
  private PluginDataEntryDao pluginDataEntryDao;

  @Autowired
  public PluginService(final PluginDao pluginDao, final PluginDataEntryDao pluginDataEntryDao) {
    this.pluginDao = pluginDao;
    this.pluginDataEntryDao = pluginDataEntryDao;
  }

  @Override
  public Plugin getByHash(final String hash) {
    Plugin result = pluginDao.getByHash(hash);
    if (result != null) {
      Hibernate.initialize(result.getUrls());
    }
    return result;
  }

  @Override
  public List<Plugin> getAll() {
    List<Plugin> result = pluginDao.getAll();
    for (final Plugin plugin : result) {
      Hibernate.initialize(plugin.getUrls());
    }
    return result;
  }

  @Override
  public PluginDataEntry getEntryByKey(final Plugin plugin, final String key, final User user) {
    PluginDataEntry result = pluginDataEntryDao.getByKey(plugin, key, user);
    if (result != null) {
      Hibernate.initialize(result.getUser());
    }
    return result;
  }

  @Override
  public void delete(final PluginDataEntry entry) {
    pluginDataEntryDao.delete(entry);
  }

  @Override
  public void delete(final Plugin plugin) {
    List<PluginDataEntry> entries = pluginDataEntryDao.getByPlugin(plugin);
    for (final PluginDataEntry pluginDataEntry : entries) {
      pluginDataEntryDao.delete(pluginDataEntry);
    }
    pluginDao.delete(plugin);
  }

  @Override
  public Plugin add(final Plugin plugin) {
    return pluginDao.add(plugin);
  }

  @Override
  public void add(final PluginDataEntry entry) {
    pluginDataEntryDao.add(entry);
  }

  @Override
  public void update(final PluginDataEntry entry) {
    pluginDataEntryDao.update(entry);
  }

  @Override
  public void update(final Plugin plugin) {
    pluginDao.update(plugin);
  }

}
