package lcsb.mapviewer.services.impl;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import org.hibernate.Hibernate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.kinetics.SbmlParameter;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.persist.dao.map.ReactionDao;
import lcsb.mapviewer.persist.dao.map.ReactionProperty;
import lcsb.mapviewer.services.ObjectNotFoundException;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.interfaces.IReactionService;

@Transactional
@Service
public class ReactionService implements IReactionService {

  @Autowired
  private IModelService modelService;

  @Autowired
  private ReactionDao reactionDao;

  @Override
  public Reaction getById(final int id) {
    return reactionDao.getById(id);
  }

  @Override
  public Reaction getReactionById(final String projectId, final int mapId, final int elementId) throws ObjectNotFoundException {
    ModelData map = modelService.getModelByMapId(projectId, mapId);
    return reactionDao.getById(elementId, map);
  }

  @Override
  public List<Reaction> getReactionById(final String projectId, final String mapId, final Set<Integer> ids,
      final Set<Integer> participantIds) throws ObjectNotFoundException {
    List<ModelData> maps = modelService.getModelsByMapId(projectId, mapId);
    return reactionDao.getByIds(ids, participantIds, maps);
  }

  @Override
  public List<SbmlParameter> getParameters(final String projectId, final String mapId) throws ObjectNotFoundException {
    List<ModelData> maps = modelService.getModelsByMapId(projectId, mapId);
    return reactionDao.getParametersByMaps(maps);
  }

  @Override
  public Map<MiriamType, Integer> getAnnotationStatistics(final String projectId, final String mapId) throws ObjectNotFoundException {
    Map<MiriamType, Integer> elementAnnotations = new TreeMap<>();
    Map<MiriamType, Integer> data = reactionDao
        .getAnnotationStatistics(modelService.getModelsByMapId(projectId, mapId));
    for (final MiriamType mt : MiriamType.values()) {
      if (data.get(mt) != null) {
        elementAnnotations.put(mt, data.get(mt));
      } else {
        elementAnnotations.put(mt, 0);
      }
    }
    return elementAnnotations;
  }

  @Override
  public Page<Reaction> getReactionsByFilter(final Map<ReactionProperty, List<? extends Object>> reactionSearch, final boolean initializeLazy) {
    Page<Reaction> result = reactionDao.getByFilter(Pageable.unpaged(), reactionSearch);
    if (initializeLazy) {
      for (final Reaction reaction : result) {
        Hibernate.initialize(reaction.getModelData());
      }
    }
    return result;
  }

}
