package lcsb.mapviewer.services.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.common.exception.InvalidArgumentException;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.kinetics.SbmlFunction;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.persist.dao.ProjectDao;
import lcsb.mapviewer.persist.dao.map.ModelDao;
import lcsb.mapviewer.persist.dao.map.kinetics.SbmlFunctionDao;
import lcsb.mapviewer.services.ObjectNotFoundException;
import lcsb.mapviewer.services.interfaces.ISbmlFunctionService;

@Transactional
@Service
public class SbmlFunctionService implements ISbmlFunctionService {

  @Autowired
  private ProjectDao projectDao;

  @Autowired
  private ModelDao modelDao;

  @Autowired
  private SbmlFunctionDao functionDao;

  @Override
  public List<SbmlFunction> getFunctionsByProjectId(final String projectId, final String mapId) throws ObjectNotFoundException {
    Project project = projectDao.getProjectByProjectId(projectId);
    if (project == null) {
      throw new ObjectNotFoundException("Project with given id does not exist");
    }

    if (Objects.equals("*", mapId)) {
      return functionDao.getByProject(project);
    } else {
      try {
        ModelData map = project.getMapById(mapId);
        if (map == null || !map.getProject().getProjectId().equals(projectId)) {
          throw new ObjectNotFoundException("Map with given id does not exist");
        } else {
          return new ArrayList<>(map.getFunctions());
        }
      } catch (final InvalidArgumentException e) {
        throw new ObjectNotFoundException(e);
      }
    }
  }

  @Override
  public SbmlFunction getFunctionById(final String projectId, final int mapId, final int functionId) throws ObjectNotFoundException {
    ModelData map = modelDao.getById(projectId, mapId);
    if (map == null) {
      throw new ObjectNotFoundException("Map with given id does not exist");
    }
    return functionDao.getById(functionId, map);
  }
}
